local L = Rock("LibRockLocale-1.0"):GetTranslationNamespace("AbramsFu")

AbramsFu = Rock:NewAddon("AbramsFu", "LibFuBarPlugin-3.0", "LibRockConfig-1.0", "LibRockDB-1.0", "LibRockEvent-1.0")

AbramsFu:SetDatabase("AbramsFuDB")
AbramsFu:SetDatabaseDefaults('profile', {
	ar = 0,
	ag = 1,
	ab = 0,
	drr = 0,
	drg = .5,
	drb = 1,
	hr = 0,
	hg = .5,
	hb = 1,
	lr = 0,
	lg = 1,
	lb = 0,
	vr = 1,
	vg = 1,
	vb = 1,
})

AbramsFu.version = "1.0." .. string.sub("$Revision: 51516 $", 12, -3)
AbramsFu.date = string.sub("$Date: 2007-10-09 15:10:30 -0400 (Tue, 09 Oct 2007) $", 8, 17)
AbramsFu:SetFuBarOption('clickableTooltip', false)
AbramsFu:SetFuBarOption('defaultPosition', "CENTER")
AbramsFu:SetFuBarOption('hasIcon', false)
AbramsFu:SetFuBarOption('tooltipType',"GameTooltip")

L:AddTranslations("enUS", function() return {
  ["name"] = "AbramsFu",
  ["desc"] = "Displays a variety of useful stats for tanking",
  ["avcn"] = "Avoidance Color",
  ["avcd"] = "Set the color of the avoidance percentage",
  ["drcn"] = "DR Color",
  ["drcd"] = "Set the color of the damage reduction percentage",
  ["hcn"] = "Header Color",
  ["hcd"] = "Set the color of header text",
  ["lcn"] = "Label Color",
  ["lcd"] = "Set the color of label text",
  ["vcn"] = "Value Color",
  ["vcd"] = "Set the color of value text",
  ["defcalc"] = "Defense Calculation",
  ["tdef"] = "Total Defense",
  ["bdef"] = "Base Defense",
  ["abon"] = "Armor Bonus",
  ["anti"] = "Anticipation Bonus",
  ["ct"] = "Combat Table",
  ["bc"] = "Block Chance",
  ["pc"] = "Parry Chance",
  ["dc"] = "Dodge Chance",
  ["armor"] = "Armor",
  ["ba"] = "Base Armor",
  ["ak"] = "Armor Kits/Enchants",
  ["ab"] = "Armor Buffs",
  ["ad"] = "Armor Debuffs",
  ["ta"] = "Total Armor",
  ["dr"] = "Damage Reduction",
  ["badr"] = "Base Armor DR",
} end)

function AbramsFu:OnInitialize()
  local optionsTable = {
    name = L["name"],
    desc = L["desc"],
    type = 'group',
	  handler = AbramsFu,
    args = {
		  avoidcolor = {
			  type = 'color',
			  name = AVOIDCOLOR or L["avcn"],
			  desc = L["avcd"],
			  get = "GetAvoidanceColor",
			  set = "SetAvoidanceColor",
			  disabled = function()
				  return not AbramsFu:IsFuBarTextColored()
		    end
		  },
		  drcolor = {
			  type = 'color',
			  name = DRCOLOR or L["drcn"],
			  desc = L["avcd"],
			  get = "GetDRColor",
			  set = "SetDRColor",
			  disabled = function()
				  return not AbramsFu:IsFuBarTextColored()
		    end
		  },
		  headcolor = {
			  type = 'color',
			  name = HEADCOLOR or L["hcn"],
			  desc = L["hcd"],
			  get = "GetHeaderColor",
			  set = "SetHeaderColor",
			  disabled = function()
				  return not AbramsFu:IsFuBarTextColored()
		    end
		  },
		  labelcolor = {
			  type = 'color',
			  name = LABELCOLOR or L["lcn"],
			  desc = L["lcd"],
			  get = "GetLabelColor",
			  set = "SetLabelColor",
			  disabled = function()
				  return not AbramsFu:IsFuBarTextColored()
		    end
		  },
		  valuecolor = {
			  type = 'color',
			  name = VALUECOLOR or L["vcn"],
			  desc = L["vcd"],
			  get = "GetValueColor",
			  set = "SetValueColor",
			  disabled = function()
				  return not AbramsFu:IsFuBarTextColored()
		    end
		  }
    }
  }
  AbramsFu:SetConfigTable(optionsTable)
  AbramsFu.OnMenuRequest = optionsTable
end

function AbramsFu:OnEnable()
  self:AddEventListener("Blizzard","PLAYER_ENTERING_WORLD","UpdateData")
  self:AddEventListener("Blizzard","CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS","UpdateData")
  self:AddEventListener("Blizzard","CHAT_MSG_SPELL_AURA_GONE_SELF","UpdateData")
  self:AddEventListener("Blizzard","UNIT_INVENTORY_CHANGED","UpdateData")
  self:AddEventListener("Blizzard","PLAYER_TARGET_CHANGED","UpdateData")
  self:AddEventListener("Blizzard","CHARACTER_POINTS_CHANGED","UpdateData")
  self:AddEventListener("Blizzard","PLAYER_LEVEL_UP","UpdateData") 
end

function AbramsFu:OnDisable()
  self:RemoveAllEventListeners()
end

function AbramsFu:UpdateData(namespace, event)
  self:UpdateFuBarText()
end

function AbramsFu:OnUpdateFuBarText()
  self:SetFuBarText(string.format("|cff%s%.2f%%|r |cff%s%.2f%%|r",self:GetAvoidanceHexColor(),
    self:GetAvoidance(),self:GetDRHexColor(),self:GetDR()))
end

function AbramsFu:OnUpdateFuBarTooltip()
  local baseDefense, armorDefense = UnitDefense("player")
  local totalDefense = baseDefense + armorDefense
  local anticipation = self:GetAnticipationBonus()
  local baseArmor, totalArmor, nobuffArmor, armorBuffs, armorDebuffs = UnitArmor("player")
  local armorEnhancements = nobuffArmor - baseArmor
  
  baseDefense = baseDefense - anticipation
  
  GameTooltip:AddLine(L["name"])
  GameTooltip:AddLine(" ")

  GameTooltip:AddLine(L["defcalc"],self.db.profile.hr,self.db.profile.hg,self.db.profile.hb)
  if baseDefense == totalDefense then
    GameTooltip:AddDoubleLine(L["tdef"],string.format("%d",totalDefense),self.db.profile.lr, 
      self.db.profile.lg, self.db.profile.lb, self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  else
    GameTooltip:AddDoubleLine(L["bdef"],string.format("%d",baseDefense),self.db.profile.lr, 
      self.db.profile.lg, self.db.profile.lb, self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    if armorDefense > 0 then
      GameTooltip:AddDoubleLine(L["abon"],string.format("+%d",armorDefense),
        self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
        self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    end
    if anticipation > 0 then
      GameTooltip:AddDoubleLine(L["anti"],string.format("+%d",anticipation),
        self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
        self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    end
    GameTooltip:AddDoubleLine(L["tdef"],string.format("%d",totalDefense),self.db.profile.lr, 
      self.db.profile.lg, self.db.profile.lb, self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  end

  GameTooltip:AddLine(" ")
  GameTooltip:AddLine(L["ct"],self.db.profile.hr,self.db.profile.hg,self.db.profile.hb)
  GameTooltip:AddDoubleLine(L["dc"],string.format("%.2f%%",GetDodgeChance()),
    self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
    self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  GameTooltip:AddDoubleLine(L["bc"],string.format("%.2f%%",GetBlockChance()),
    self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
    self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  GameTooltip:AddDoubleLine(L["pc"],string.format("%.2f%%",GetParryChance()),
    self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
    self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)

  GameTooltip:AddLine(" ")
  GameTooltip:AddLine(L["armor"],self.db.profile.hr,self.db.profile.hg,self.db.profile.hb)
  if baseArmor == totalArmor then
    GameTooltip:AddDoubleLine(L["armor"],string.format("%d",totalArmor),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
      self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  else
    GameTooltip:AddDoubleLine(L["ba"],string.format("%d",baseArmor),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
      self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    if armorEnhancements > 0 then
      GameTooltip:AddDoubleLine(L["ak"],string.format("+%d",armorEnhancements),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
        self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    end
    if armorBuffs > 0 then
      GameTooltip:AddDoubleLine(L["ab"],string.format("+%d",armorBuffs),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
        self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    end
    if armorDebuffs < 0 then
      GameTooltip:AddDoubleLine(L["ad"],string.format("%d",armorDebuffs),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
        self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
    end
    GameTooltip:AddDoubleLine(L["ta"],string.format("%d",totalArmor),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
      self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  end

  GameTooltip:AddLine(" ")
  GameTooltip:AddLine(L["dr"],self.db.profile.hr,self.db.profile.hg,self.db.profile.hb)
  GameTooltip:AddDoubleLine(L["badr"],string.format("%.2f%%",self:GetBaseDR()),self.db.profile.lr, self.db.profile.lg, self.db.profile.lb, 
    self.db.profile.vr, self.db.profile.vg, self.db.profile.vb)
  
end

function AbramsFu:GetAvoidance()
	local baseDefense, armorDefense = UnitDefense("player")
	local totalDefense = baseDefense + armorDefense
	local maxDefense = self:GetMaxDefense()
	local avoid = (GetBlockChance() + GetParryChance() + GetDodgeChance())
	if totalDefense and maxDefense and (totalDefense > maxDefense) then
		avoid = avoid + (((totalDefense - maxDefense) + self:GetAnticipationBonus())*0.04 + 5)
	end
	return avoid
end

function AbramsFu:GetMaxDefense()
  local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier 
  local skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription
  local skillIndex
	for skillIndex = 1, GetNumSkillLines() do
		skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription = GetSkillLineInfo(skillIndex)
		if skillName == "Defense" then
			return skillMaxRank
		end
	end
end

function AbramsFu:GetAnticipationBonus()
  local bonus = 0
	local ucClass, ucEngClass = UnitClass("player")
	local nameTalent, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq
  if (ucEngClass == "PALADIN") then
	  nameTalent, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(2, 9)
	  if (currentRank > 0) then
		  bonus = 4*currentRank
	  end
	elseif (ucEngClass == "WARRIOR") then
		nameTalent, iconPath, tier, column, currentRank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(3, 3)
		if currentRank > 0 then
  		bonus = 4*currentRank
  	end
  end
  return bonus
end

function AbramsFu:GetDR()
  local dr = 0 
  local baseDR = self:GetBaseDR()
  dr = baseDR
  return dr
end

function AbramsFu:GetBaseDR()
  local dr = 0
  local level = UnitLevel("player")
  local mobLevel = UnitLevel("target")
  local baseArmor, totalArmor, nobuffArmor, armorBuffs, armorDebuffs = UnitArmor("player")
  if mobLevel == 0 then
    mobLevel = level
  end
  if level < 60 then
    dr = (totalArmor / (totalArmor+400+85*mobLevel)) * 100
  else
    dr = (totalArmor / (totalArmor+400+85*(mobLevel+4.5*(mobLevel-59)))) * 100
  end
  return dr
end

function AbramsFu:GetHexColor(r,g,b)
  return string.format("%02x%02x%02x", r * 255, g * 255, b * 255)
end

function AbramsFu:GetAvoidanceColor()
	return self.db.profile.ar, self.db.profile.ag, self.db.profile.ab
end

function AbramsFu:SetAvoidanceColor(r, g, b)
	self.db.profile.ar = r
	self.db.profile.ag = g
	self.db.profile.ab = b
	self:UpdateFuBarText()
end

function AbramsFu:GetAvoidanceHexColor()
	return self:GetHexColor(self.db.profile.ar, self.db.profile.ag, self.db.profile.ab)
end

function AbramsFu:GetDRColor()
	return self.db.profile.drr, self.db.profile.drg, self.db.profile.drb
end

function AbramsFu:SetDRColor(r, g, b)
	self.db.profile.drr = r
	self.db.profile.drg = g
	self.db.profile.drb = b
	self:UpdateFuBarText()
end

function AbramsFu:GetDRHexColor()
	return self:GetHexColor(self.db.profile.drr, self.db.profile.drg, self.db.profile.drb)
end

function AbramsFu:GetHeaderColor()
	return self.db.profile.hr, self.db.profile.hg, self.db.profile.hb
end

function AbramsFu:SetHeaderColor(r, g, b)
	self.db.profile.hr = r
	self.db.profile.hg = g
	self.db.profile.hb = b
end

function AbramsFu:GetLabelColor()
	return self.db.profile.lr, self.db.profile.hg, self.db.profile.lb
end

function AbramsFu:SetLabelColor(r, g, b)
	self.db.profile.lr = r
	self.db.profile.lg = g
	self.db.profile.lb = b
end

function AbramsFu:GetValueColor()
	return self.db.profile.vr, self.db.profile.vg, self.db.profile.vb
end

function AbramsFu:SetValueColor(r, g, b)
	self.db.profile.vr = r
	self.db.profile.vg = g
	self.db.profile.vb = b
end


